@@ -76,9 +76,13 @@ public class AvroConverters {
       }
     }
 
+    public T prepareDictionaryValue(T value) {
+      return value;
+    }
+
     @Override
     public void addValueFromDictionary(int dictionaryId) {
-      parent.add(dict[dictionaryId]);
+      parent.add(prepareDictionaryValue(dict[dictionaryId]));
     }
   }
 
@@ -220,6 +224,11 @@ public class AvroConverters {
     public ByteBuffer convert(Binary binary) {
       return ByteBuffer.wrap(binary.getBytes());
     }
+
+    @Override
+    public ByteBuffer prepareDictionaryValue(ByteBuffer value) {
+      return value.duplicate();
+    }
   }
 
   static final class FieldStringConverter extends BinaryConverter<String> {
