@@ -1,4 +1,4 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- * 
+ *
  *   http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -18,14 +18,15 @@
  */
 package org.apache.parquet.avro;
 
-import com.google.common.base.Charsets;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
 import com.google.common.io.Resources;
 import java.io.File;
+import java.io.IOException;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.nio.ByteBuffer;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
@@ -37,6 +38,7 @@ import java.util.Random;
 import org.apache.avro.Conversions;
 import org.apache.avro.LogicalTypes;
 import org.apache.avro.Schema;
+import org.apache.avro.SchemaBuilder;
 import org.apache.avro.generic.GenericData;
 import org.apache.avro.generic.GenericFixed;
 import org.apache.avro.generic.GenericRecord;
@@ -87,29 +89,27 @@ public class TestReadWrite {
     Schema schema = new Schema.Parser().parse(
         Resources.getResource("array.avsc").openStream());
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
+    // Write a record with an empty array.
+    List<Integer> emptyArray = new ArrayList<>();
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    Path file = new Path(createTempFile().getPath());
+
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(file)
         .withSchema(schema)
         .withConf(testConf)
-        .build();
-
-    // Write a record with an empty array.
-    List<Integer> emptyArray = new ArrayList<Integer>();
-    GenericData.Record record = new GenericRecordBuilder(schema)
+        .build()) {
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("myarray", emptyArray).build();
-    writer.write(record);
-    writer.close();
+      writer.write(record);
+    }
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
+    try (AvroParquetReader<GenericRecord> reader = new AvroParquetReader<>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
 
-    assertNotNull(nextRecord);
-    assertEquals(emptyArray, nextRecord.get("myarray"));
+      assertNotNull(nextRecord);
+      assertEquals(emptyArray, nextRecord.get("myarray"));
+    }
   }
 
   @Test
@@ -117,29 +117,27 @@ public class TestReadWrite {
     Schema schema = new Schema.Parser().parse(
         Resources.getResource("map.avsc").openStream());
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
+    Path file = new Path(createTempFile().getPath());
+    ImmutableMap<String, Integer> emptyMap = new ImmutableMap.Builder<String, Integer>().build();
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(file)
         .withSchema(schema)
         .withConf(testConf)
-        .build();
+        .build()) {
 
-    // Write a record with an empty map.
-    ImmutableMap emptyMap = new ImmutableMap.Builder<String, Integer>().build();
-    GenericData.Record record = new GenericRecordBuilder(schema)
+      // Write a record with an empty map.
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("mymap", emptyMap).build();
-    writer.write(record);
-    writer.close();
+      writer.write(record);
+    }
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
 
-    assertNotNull(nextRecord);
-    assertEquals(emptyMap, nextRecord.get("mymap"));
+      assertNotNull(nextRecord);
+      assertEquals(emptyMap, nextRecord.get("mymap"));
+    }
   }
 
   @Test
@@ -147,33 +145,31 @@ public class TestReadWrite {
     Schema schema = new Schema.Parser().parse(
         Resources.getResource("map_with_nulls.avsc").openStream());
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
-
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
-        .<GenericRecord>builder(file)
-        .withSchema(schema)
-        .withConf(testConf)
-        .build();
+    Path file = new Path(createTempFile().getPath());
 
     // Write a record with a null value
-    Map<CharSequence, Integer> map = new HashMap<CharSequence, Integer>();
+    Map<CharSequence, Integer> map = new HashMap<>();
     map.put(str("thirty-four"), 34);
     map.put(str("eleventy-one"), null);
     map.put(str("one-hundred"), 100);
 
-    GenericData.Record record = new GenericRecordBuilder(schema)
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
+        .<GenericRecord>builder(file)
+        .withSchema(schema)
+        .withConf(testConf)
+        .build()) {
+
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("mymap", map).build();
-    writer.write(record);
-    writer.close();
+      writer.write(record);
+    }
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
 
-    assertNotNull(nextRecord);
-    assertEquals(map, nextRecord.get("mymap"));
+      assertNotNull(nextRecord);
+      assertEquals(map, nextRecord.get("mymap"));
+    }
   }
 
   @Test(expected=RuntimeException.class)
@@ -182,26 +178,24 @@ public class TestReadWrite {
     schema.setFields(Lists.newArrayList(
         new Schema.Field("mymap", Schema.createMap(Schema.create(Schema.Type.INT)), null, null)));
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
+    Path file = new Path(createTempFile().getPath());
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(file)
         .withSchema(schema)
         .withConf(testConf)
-        .build();
+        .build()) {
 
-    // Write a record with a null value
-    Map<String, Integer> map = new HashMap<String, Integer>();
-    map.put("thirty-four", 34);
-    map.put("eleventy-one", null);
-    map.put("one-hundred", 100);
+      // Write a record with a null value
+      Map<String, Integer> map = new HashMap<String, Integer>();
+      map.put("thirty-four", 34);
+      map.put("eleventy-one", null);
+      map.put("one-hundred", 100);
 
-    GenericData.Record record = new GenericRecordBuilder(schema)
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("mymap", map).build();
-    writer.write(record);
+      writer.write(record);
+    }
   }
 
   @Test
@@ -209,29 +203,27 @@ public class TestReadWrite {
     Schema schema = new Schema.Parser().parse(
         Resources.getResource("map.avsc").openStream());
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
+    Path file = new Path(createTempFile().getPath());
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(file)
         .withSchema(schema)
         .withConf(testConf)
-        .build();
+        .build()) {
 
-    // Write a record with a map with Utf8 keys.
-    GenericData.Record record = new GenericRecordBuilder(schema)
+      // Write a record with a map with Utf8 keys.
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("mymap", ImmutableMap.of(new Utf8("a"), 1, new Utf8("b"), 2))
         .build();
-    writer.write(record);
-    writer.close();
+      writer.write(record);
+    }
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
 
-    assertNotNull(nextRecord);
-    assertEquals(ImmutableMap.of(str("a"), 1, str("b"), 2), nextRecord.get("mymap"));
+      assertNotNull(nextRecord);
+      assertEquals(ImmutableMap.of(str("a"), 1, str("b"), 2), nextRecord.get("mymap"));
+    }
   }
 
   @Rule
@@ -252,37 +244,37 @@ public class TestReadWrite {
     File file = temp.newFile("decimal.parquet");
     file.delete();
     Path path = new Path(file.toString());
+    List<GenericRecord> expected = Lists.newArrayList();
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(path)
         .withDataModel(decimalSupport)
         .withSchema(decimalSchema)
-        .build();
+        .build()) {
 
-    Random random = new Random(34L);
-    GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
-    List<GenericRecord> expected = Lists.newArrayList();
-    for (int i = 0; i < 1000; i += 1) {
-      BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
-      builder.set("dec", dec);
+      Random random = new Random(34L);
+      GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
+      for (int i = 0; i < 1000; i += 1) {
+        BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
+        builder.set("dec", dec);
 
-      GenericRecord rec = builder.build();
-      expected.add(rec);
-      writer.write(builder.build());
+        GenericRecord rec = builder.build();
+        expected.add(rec);
+        writer.write(builder.build());
+      }
     }
-    writer.close();
+    List<GenericRecord> records = Lists.newArrayList();
 
-    ParquetReader<GenericRecord> reader = AvroParquetReader
+    try(ParquetReader<GenericRecord> reader = AvroParquetReader
         .<GenericRecord>builder(path)
         .withDataModel(decimalSupport)
         .disableCompatibility()
-        .build();
-    List<GenericRecord> records = Lists.newArrayList();
-    GenericRecord rec;
-    while ((rec = reader.read()) != null) {
-      records.add(rec);
+        .build()) {
+      GenericRecord rec;
+      while ((rec = reader.read()) != null) {
+        records.add(rec);
+      }
     }
-    reader.close();
 
     Assert.assertTrue("dec field should be a BigDecimal instance",
         records.get(0).get("dec") instanceof BigDecimal);
@@ -304,37 +296,37 @@ public class TestReadWrite {
     File file = temp.newFile("decimal.parquet");
     file.delete();
     Path path = new Path(file.toString());
+    List<GenericRecord> expected = Lists.newArrayList();
 
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(path)
         .withDataModel(decimalSupport)
         .withSchema(decimalSchema)
-        .build();
+        .build()) {
 
-    Random random = new Random(34L);
-    GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
-    List<GenericRecord> expected = Lists.newArrayList();
-    for (int i = 0; i < 1000; i += 1) {
-      BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
-      builder.set("dec", dec);
+      Random random = new Random(34L);
+      GenericRecordBuilder builder = new GenericRecordBuilder(decimalSchema);
+      for (int i = 0; i < 1000; i += 1) {
+        BigDecimal dec = new BigDecimal(new BigInteger(31, random), 2);
+        builder.set("dec", dec);
 
-      GenericRecord rec = builder.build();
-      expected.add(rec);
-      writer.write(builder.build());
+        GenericRecord rec = builder.build();
+        expected.add(rec);
+        writer.write(builder.build());
+      }
     }
-    writer.close();
+    List<GenericRecord> records = Lists.newArrayList();
 
-    ParquetReader<GenericRecord> reader = AvroParquetReader
+    try(ParquetReader<GenericRecord> reader = AvroParquetReader
         .<GenericRecord>builder(path)
         .withDataModel(decimalSupport)
         .disableCompatibility()
-        .build();
-    List<GenericRecord> records = Lists.newArrayList();
-    GenericRecord rec;
-    while ((rec = reader.read()) != null) {
-      records.add(rec);
+        .build()) {
+      GenericRecord rec;
+      while ((rec = reader.read()) != null) {
+        records.add(rec);
+      }
     }
-    reader.close();
 
     Assert.assertTrue("dec field should be a BigDecimal instance",
         records.get(0).get("dec") instanceof BigDecimal);
@@ -346,46 +338,39 @@ public class TestReadWrite {
     Schema schema = new Schema.Parser().parse(
         Resources.getResource("all.avsc").openStream());
 
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
-    
-    ParquetWriter<GenericRecord> writer = AvroParquetWriter
+    Path file = new Path(createTempFile().getPath());
+    List<Integer> integerArray = Arrays.asList(1, 2, 3);
+    GenericData.Record nestedRecord = new GenericRecordBuilder(
+      schema.getField("mynestedrecord").schema())
+      .set("mynestedint", 1).build();
+    List<Integer> emptyArray = new ArrayList<Integer>();
+    Schema arrayOfOptionalIntegers = Schema.createArray(
+      optional(Schema.create(Schema.Type.INT)));
+    GenericData.Array<Integer> genericIntegerArrayWithNulls =
+      new GenericData.Array<Integer>(
+        arrayOfOptionalIntegers,
+        Arrays.asList(1, null, 2, null, 3));
+    GenericFixed genericFixed = new GenericData.Fixed(
+      Schema.createFixed("fixed", null, null, 1), new byte[]{(byte) 65});
+    ImmutableMap<String, Integer> emptyMap = new ImmutableMap.Builder<String, Integer>().build();
+
+    try(ParquetWriter<GenericRecord> writer = AvroParquetWriter
         .<GenericRecord>builder(file)
         .withSchema(schema)
         .withConf(testConf)
-        .build();
+        .build()) {
 
-    GenericData.Record nestedRecord = new GenericRecordBuilder(
-        schema.getField("mynestedrecord").schema())
-            .set("mynestedint", 1).build();
-
-    List<Integer> integerArray = Arrays.asList(1, 2, 3);
-    GenericData.Array<Integer> genericIntegerArray = new GenericData.Array<Integer>(
+      GenericData.Array<Integer> genericIntegerArray = new GenericData.Array<Integer>(
         Schema.createArray(Schema.create(Schema.Type.INT)), integerArray);
 
-    GenericFixed genericFixed = new GenericData.Fixed(
-        Schema.createFixed("fixed", null, null, 1), new byte[] { (byte) 65 });
-
-    List<Integer> emptyArray = new ArrayList<Integer>();
-    ImmutableMap emptyMap = new ImmutableMap.Builder<String, Integer>().build();
-
-    Schema arrayOfOptionalIntegers = Schema.createArray(
-        optional(Schema.create(Schema.Type.INT)));
-    GenericData.Array<Integer> genericIntegerArrayWithNulls =
-        new GenericData.Array<Integer>(
-            arrayOfOptionalIntegers,
-            Arrays.asList(1, null, 2, null, 3));
-
-    GenericData.Record record = new GenericRecordBuilder(schema)
+      GenericData.Record record = new GenericRecordBuilder(schema)
         .set("mynull", null)
         .set("myboolean", true)
         .set("myint", 1)
         .set("mylong", 2L)
         .set("myfloat", 3.1f)
         .set("mydouble", 4.1)
-        .set("mybytes", ByteBuffer.wrap("hello".getBytes(Charsets.UTF_8)))
+        .set("mybytes", ByteBuffer.wrap("hello".getBytes(StandardCharsets.UTF_8)))
         .set("mystring", "hello")
         .set("mynestedrecord", nestedRecord)
         .set("myenum", "a")
@@ -398,11 +383,13 @@ public class TestReadWrite {
         .set("myfixed", genericFixed)
         .build();
 
-    writer.write(record);
-    writer.close();
+      writer.write(record);
+    }
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
+    final GenericRecord nextRecord;
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file)) {
+      nextRecord = reader.read();
+    }
 
     Object expectedEnumSymbol = compat ? "a" :
         new GenericData.EnumSymbol(schema.getField("myenum").schema(), "a");
@@ -414,7 +401,7 @@ public class TestReadWrite {
     assertEquals(2L, nextRecord.get("mylong"));
     assertEquals(3.1f, nextRecord.get("myfloat"));
     assertEquals(4.1, nextRecord.get("mydouble"));
-    assertEquals(ByteBuffer.wrap("hello".getBytes(Charsets.UTF_8)), nextRecord.get("mybytes"));
+    assertEquals(ByteBuffer.wrap("hello".getBytes(StandardCharsets.UTF_8)), nextRecord.get("mybytes"));
     assertEquals(str("hello"), nextRecord.get("mystring"));
     assertEquals(expectedEnumSymbol, nextRecord.get("myenum"));
     assertEquals(nestedRecord, nextRecord.get("mynestedrecord"));
@@ -429,13 +416,10 @@ public class TestReadWrite {
 
   @Test
   public void testAllUsingDefaultAvroSchema() throws Exception {
-    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
-    tmp.deleteOnExit();
-    tmp.delete();
-    Path file = new Path(tmp.getPath());
+    Path file = new Path(createTempFile().getPath());
 
     // write file using Parquet APIs
-    ParquetWriter<Map<String, Object>> parquetWriter = new ParquetWriter<Map<String, Object>>(file,
+    try(ParquetWriter<Map<String, Object>> parquetWriter = new ParquetWriter<>(file,
         new WriteSupport<Map<String, Object>>() {
 
       private RecordConsumer recordConsumer;
@@ -555,7 +539,7 @@ public class TestReadWrite {
 
         recordConsumer.startField("mymap", index);
         recordConsumer.startGroup();
-        recordConsumer.startField("map", 0);
+        recordConsumer.startField("key_value", 0);
         recordConsumer.startGroup();
         Map<String, Integer> mymap = (Map<String, Integer>) record.get("mymap");
         recordConsumer.startField("key", 0);
@@ -569,7 +553,7 @@ public class TestReadWrite {
         }
         recordConsumer.endField("value", 1);
         recordConsumer.endGroup();
-        recordConsumer.endField("map", 0);
+        recordConsumer.endField("key_value", 0);
         recordConsumer.endGroup();
         recordConsumer.endField("mymap", index++);
 
@@ -579,26 +563,26 @@ public class TestReadWrite {
 
         recordConsumer.endMessage();
       }
-    });
-    Map<String, Object> record = new HashMap<String, Object>();
-    record.put("myboolean", true);
-    record.put("myint", 1);
-    record.put("mylong", 2L);
-    record.put("myfloat", 3.1f);
-    record.put("mydouble", 4.1);
-    record.put("mybytes", ByteBuffer.wrap("hello".getBytes(Charsets.UTF_8)));
-    record.put("mystring", "hello");
-    record.put("myenum", "a");
-    record.put("mynestedint", 1);
-    record.put("myarray", new int[] {1, 2, 3});
-    record.put("myoptionalarray", new int[]{1, 2, 3});
-    record.put("myarrayofoptional", new Integer[] {1, null, 2, null, 3});
-    record.put("myrecordarraya", new int[] {1, 2, 3});
-    record.put("myrecordarrayb", new int[] {4, 5, 6});
-    record.put("mymap", ImmutableMap.of("a", 1, "b", 2));
-    record.put("myfixed", new byte[] { (byte) 65 });
-    parquetWriter.write(record);
-    parquetWriter.close();
+    })) {
+      Map<String, Object> record = new HashMap<String, Object>();
+      record.put("myboolean", true);
+      record.put("myint", 1);
+      record.put("mylong", 2L);
+      record.put("myfloat", 3.1f);
+      record.put("mydouble", 4.1);
+      record.put("mybytes", ByteBuffer.wrap("hello".getBytes(StandardCharsets.UTF_8)));
+      record.put("mystring", "hello");
+      record.put("myenum", "a");
+      record.put("mynestedint", 1);
+      record.put("myarray", new int[]{1, 2, 3});
+      record.put("myoptionalarray", new int[]{1, 2, 3});
+      record.put("myarrayofoptional", new Integer[]{1, null, 2, null, 3});
+      record.put("myrecordarraya", new int[]{1, 2, 3});
+      record.put("myrecordarrayb", new int[]{4, 5, 6});
+      record.put("mymap", ImmutableMap.of("a", 1, "b", 2));
+      record.put("myfixed", new byte[]{(byte) 65});
+      parquetWriter.write(record);
+    }
 
     Schema nestedRecordSchema = Schema.createRecord("mynestedrecord", null, null, false);
     nestedRecordSchema.setFields(Arrays.asList(
@@ -626,25 +610,128 @@ public class TestReadWrite {
     GenericFixed genericFixed = new GenericData.Fixed(
         Schema.createFixed("fixed", null, null, 1), new byte[] { (byte) 65 });
 
-    AvroParquetReader<GenericRecord> reader = new AvroParquetReader<GenericRecord>(testConf, file);
-    GenericRecord nextRecord = reader.read();
-    assertNotNull(nextRecord);
-    assertEquals(true, nextRecord.get("myboolean"));
-    assertEquals(1, nextRecord.get("myint"));
-    assertEquals(2L, nextRecord.get("mylong"));
-    assertEquals(3.1f, nextRecord.get("myfloat"));
-    assertEquals(4.1, nextRecord.get("mydouble"));
-    assertEquals(ByteBuffer.wrap("hello".getBytes(Charsets.UTF_8)), nextRecord.get("mybytes"));
-    assertEquals(str("hello"), nextRecord.get("mystring"));
-    assertEquals(str("a"), nextRecord.get("myenum")); // enum symbols are unknown
-    assertEquals(nestedRecord, nextRecord.get("mynestedrecord"));
-    assertEquals(integerArray, nextRecord.get("myarray"));
-    assertEquals(integerArray, nextRecord.get("myoptionalarray"));
-    assertEquals(ingeterArrayWithNulls, nextRecord.get("myarrayofoptional"));
-    assertEquals(genericRecordArray, nextRecord.get("myrecordarray"));
-    assertEquals(ImmutableMap.of(str("a"), 1, str("b"), 2), nextRecord.get("mymap"));
-    assertEquals(genericFixed, nextRecord.get("myfixed"));
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
+      assertNotNull(nextRecord);
+      assertEquals(true, nextRecord.get("myboolean"));
+      assertEquals(1, nextRecord.get("myint"));
+      assertEquals(2L, nextRecord.get("mylong"));
+      assertEquals(3.1f, nextRecord.get("myfloat"));
+      assertEquals(4.1, nextRecord.get("mydouble"));
+      assertEquals(ByteBuffer.wrap("hello".getBytes(StandardCharsets.UTF_8)), nextRecord.get("mybytes"));
+      assertEquals(str("hello"), nextRecord.get("mystring"));
+      assertEquals(str("a"), nextRecord.get("myenum")); // enum symbols are unknown
+      assertEquals(nestedRecord, nextRecord.get("mynestedrecord"));
+      assertEquals(integerArray, nextRecord.get("myarray"));
+      assertEquals(integerArray, nextRecord.get("myoptionalarray"));
+      assertEquals(ingeterArrayWithNulls, nextRecord.get("myarrayofoptional"));
+      assertEquals(genericRecordArray, nextRecord.get("myrecordarray"));
+      assertEquals(ImmutableMap.of(str("a"), 1, str("b"), 2), nextRecord.get("mymap"));
+      assertEquals(genericFixed, nextRecord.get("myfixed"));
+    }
+  }
 
+  @Test
+  public void testUnionWithSingleNonNullType() throws Exception {
+    Schema avroSchema = Schema.createRecord("SingleStringUnionRecord", null, null, false);
+    avroSchema.setFields(
+      Collections.singletonList(new Schema.Field("value",
+        Schema.createUnion(Schema.create(Schema.Type.STRING)), null, null)));
+
+    Path file = new Path(createTempFile().getPath());
+
+    // Parquet writer
+    try(ParquetWriter parquetWriter = AvroParquetWriter.builder(file).withSchema(avroSchema)
+      .withConf(new Configuration())
+      .build()) {
+
+      GenericRecord record = new GenericRecordBuilder(avroSchema)
+        .set("value", "theValue")
+        .build();
+
+      parquetWriter.write(record);
+    }
+
+    try(AvroParquetReader<GenericRecord> reader = new AvroParquetReader<>(testConf, file)) {
+      GenericRecord nextRecord = reader.read();
+
+      assertNotNull(nextRecord);
+      assertEquals(str("theValue"), nextRecord.get("value"));
+    }
+  }
+
+  @Test
+  public void testDuplicatedValuesWithDictionary() throws Exception {
+    Schema schema = SchemaBuilder.record("spark_schema")
+      .fields().optionalBytes("value").endRecord();
+
+    Path file = new Path(createTempFile().getPath());
+
+    String[] records = {"one", "two", "three", "three", "two", "one", "zero"};
+    try (ParquetWriter<GenericData.Record> writer = AvroParquetWriter
+      .<GenericData.Record>builder(file)
+      .withSchema(schema)
+      .withConf(testConf)
+      .build()) {
+      for (String record : records) {
+        writer.write(new GenericRecordBuilder(schema)
+          .set("value", record.getBytes()).build());
+      }
+    }
+
+    try (ParquetReader<GenericRecord> reader = AvroParquetReader
+      .<GenericRecord>builder(file)
+      .withConf(testConf).build()) {
+      GenericRecord rec;
+      int i = 0;
+      while ((rec = reader.read()) != null) {
+        ByteBuffer buf = (ByteBuffer) rec.get("value");
+        byte[] bytes = new byte[buf.remaining()];
+        buf.get(bytes);
+        assertEquals(records[i++], new String(bytes));
+      }
+    }
+  }
+
+  @Test
+  public void testNestedLists() throws Exception {
+    Schema schema = new Schema.Parser().parse(
+      Resources.getResource("nested_array.avsc").openStream());
+    Path file = new Path(createTempFile().getPath());
+
+    // Parquet writer
+    ParquetWriter parquetWriter = AvroParquetWriter.builder(file).withSchema(schema)
+      .withConf(testConf)
+      .build();
+
+    Schema innerRecordSchema = schema.getField("l1").schema().getTypes()
+      .get(1).getElementType().getTypes().get(1);
+
+    GenericRecord record = new GenericRecordBuilder(schema)
+      .set("l1", Collections.singletonList(
+        new GenericRecordBuilder(innerRecordSchema).set("l2", Collections.singletonList("hello")).build()
+      ))
+      .build();
+
+    parquetWriter.write(record);
+    parquetWriter.close();
+
+    AvroParquetReader<GenericRecord> reader = new AvroParquetReader(testConf, file);
+    GenericRecord nextRecord = reader.read();
+
+    assertNotNull(nextRecord);
+    assertNotNull(nextRecord.get("l1"));
+    List l1List = (List) nextRecord.get("l1");
+    assertNotNull(l1List.get(0));
+    List l2List = (List) ((GenericRecord) l1List.get(0)).get("l2");
+    assertEquals(str("hello"), l2List.get(0));
+  }
+
+  private File createTempFile() throws IOException {
+    File tmp = File.createTempFile(getClass().getSimpleName(), ".tmp");
+    tmp.deleteOnExit();
+    tmp.delete();
+    return tmp;
   }
 
   /**
